import { pathToRegexp } from "next/dist/compiled/path-to-regexp";
import { generateRandomActionKeyRaw } from "../app-render/action-encryption-utils";
import loadJsConfig from "../../build/load-jsconfig";
import { join, posix } from "path";
import { readFile, writeFile } from "fs/promises";
import { APP_BUILD_MANIFEST, APP_PATHS_MANIFEST, BUILD_MANIFEST, MIDDLEWARE_MANIFEST, NEXT_FONT_MANIFEST, PAGES_MANIFEST, SERVER_REFERENCE_MANIFEST, REACT_LOADABLE_MANIFEST, MIDDLEWARE_BUILD_MANIFEST, INTERCEPTION_ROUTE_REWRITE_MANIFEST, MIDDLEWARE_REACT_LOADABLE_MANIFEST } from "../../shared/lib/constants";
import { writeFileAtomic } from "../../lib/fs/write-atomic";
import { deleteCache } from "../../build/webpack/plugins/nextjs-require-cache-hot-reloader";
import { normalizeRewritesForBuildManifest, srcEmptySsgManifest } from "../../build/webpack/plugins/build-manifest-plugin";
import { isInterceptionRouteRewrite } from "../../lib/generate-interception-routes-rewrites";
import { MAGIC_IDENTIFIER_REGEX, decodeMagicIdentifier } from "../../shared/lib/magic-identifier";
import { bold, green, magenta, red } from "../../lib/picocolors";
import { HMR_ACTIONS_SENT_TO_BROWSER } from "./hot-reloader-types";
export function mergeBuildManifests(manifests) {
    const manifest = {
        pages: {
            "/_app": []
        },
        // Something in next.js depends on these to exist even for app dir rendering
        devFiles: [],
        ampDevFiles: [],
        polyfillFiles: [],
        lowPriorityFiles: [
            "static/development/_ssgManifest.js",
            "static/development/_buildManifest.js"
        ],
        rootMainFiles: [],
        ampFirstPages: []
    };
    for (const m of manifests){
        Object.assign(manifest.pages, m.pages);
        if (m.rootMainFiles.length) manifest.rootMainFiles = m.rootMainFiles;
    }
    return manifest;
}
export function mergeAppBuildManifests(manifests) {
    const manifest = {
        pages: {}
    };
    for (const m of manifests){
        Object.assign(manifest.pages, m.pages);
    }
    return manifest;
}
export function mergePagesManifests(manifests) {
    const manifest = {};
    for (const m of manifests){
        Object.assign(manifest, m);
    }
    return manifest;
}
export function mergeMiddlewareManifests(manifests) {
    const manifest = {
        version: 2,
        middleware: {},
        sortedMiddleware: [],
        functions: {}
    };
    let instrumentation = undefined;
    for (const m of manifests){
        Object.assign(manifest.functions, m.functions);
        Object.assign(manifest.middleware, m.middleware);
        if (m.instrumentation) {
            instrumentation = m.instrumentation;
        }
    }
    const updateFunctionDefinition = (fun)=>{
        return {
            ...fun,
            files: [
                ...(instrumentation == null ? void 0 : instrumentation.files) ?? [],
                ...fun.files
            ]
        };
    };
    for (const key of Object.keys(manifest.middleware)){
        const value = manifest.middleware[key];
        manifest.middleware[key] = updateFunctionDefinition(value);
    }
    for (const key of Object.keys(manifest.functions)){
        const value = manifest.functions[key];
        manifest.functions[key] = updateFunctionDefinition(value);
    }
    for (const fun of Object.values(manifest.functions).concat(Object.values(manifest.middleware))){
        for (const matcher of fun.matchers){
            if (!matcher.regexp) {
                matcher.regexp = pathToRegexp(matcher.originalSource, [], {
                    delimiter: "/",
                    sensitive: false,
                    strict: true
                }).source.replaceAll("\\/", "/");
            }
        }
    }
    manifest.sortedMiddleware = Object.keys(manifest.middleware);
    return manifest;
}
export async function mergeActionManifests(manifests) {
    const manifest = {
        node: {},
        edge: {},
        encryptionKey: await generateRandomActionKeyRaw(true)
    };
    function mergeActionIds(actionEntries, other) {
        for(const key in other){
            const action = actionEntries[key] ??= {
                workers: {},
                layer: {}
            };
            Object.assign(action.workers, other[key].workers);
            Object.assign(action.layer, other[key].layer);
        }
    }
    for (const m of manifests){
        mergeActionIds(manifest.node, m.node);
        mergeActionIds(manifest.edge, m.edge);
    }
    return manifest;
}
export function mergeFontManifests(manifests) {
    const manifest = {
        app: {},
        appUsingSizeAdjust: false,
        pages: {},
        pagesUsingSizeAdjust: false
    };
    for (const m of manifests){
        Object.assign(manifest.app, m.app);
        Object.assign(manifest.pages, m.pages);
        manifest.appUsingSizeAdjust = manifest.appUsingSizeAdjust || m.appUsingSizeAdjust;
        manifest.pagesUsingSizeAdjust = manifest.pagesUsingSizeAdjust || m.pagesUsingSizeAdjust;
    }
    return manifest;
}
export function mergeLoadableManifests(manifests) {
    const manifest = {};
    for (const m of manifests){
        Object.assign(manifest, m);
    }
    return manifest;
}
export async function getTurbopackJsConfig(dir, nextConfig) {
    const { jsConfig } = await loadJsConfig(dir, nextConfig);
    return jsConfig ?? {
        compilerOptions: {}
    };
}
export async function readPartialManifest(distDir, name, pageName, type = "pages") {
    const manifestPath = posix.join(distDir, `server`, type === "app-route" ? "app" : type, type === "middleware" || type === "instrumentation" ? "" : pageName === "/" ? "index" : pageName === "/index" || pageName.startsWith("/index/") ? `/index${pageName}` : pageName, type === "app" ? "page" : type === "app-route" ? "route" : "", name);
    return JSON.parse(await readFile(posix.join(manifestPath), "utf-8"));
}
export async function loadMiddlewareManifest(distDir, middlewareManifests, pageName, type) {
    middlewareManifests.set(pageName, await readPartialManifest(distDir, MIDDLEWARE_MANIFEST, pageName, type));
}
export async function loadBuildManifest(distDir, buildManifests, pageName, type = "pages") {
    buildManifests.set(pageName, await readPartialManifest(distDir, BUILD_MANIFEST, pageName, type));
}
async function loadAppBuildManifest(distDir, appBuildManifests, pageName) {
    appBuildManifests.set(pageName, await readPartialManifest(distDir, APP_BUILD_MANIFEST, pageName, "app"));
}
export async function loadPagesManifest(distDir, pagesManifests, pageName) {
    pagesManifests.set(pageName, await readPartialManifest(distDir, PAGES_MANIFEST, pageName));
}
async function loadAppPathManifest(distDir, appPathsManifests, pageName, type = "app") {
    appPathsManifests.set(pageName, await readPartialManifest(distDir, APP_PATHS_MANIFEST, pageName, type));
}
async function loadActionManifest(distDir, actionManifests, pageName) {
    actionManifests.set(pageName, await readPartialManifest(distDir, `${SERVER_REFERENCE_MANIFEST}.json`, pageName, "app"));
}
export async function loadFontManifest(distDir, fontManifests, pageName, type = "pages") {
    fontManifests.set(pageName, await readPartialManifest(distDir, `${NEXT_FONT_MANIFEST}.json`, pageName, type));
}
async function loadLoadableManifest(distDir, loadableManifests, pageName, type = "pages") {
    loadableManifests.set(pageName, await readPartialManifest(distDir, REACT_LOADABLE_MANIFEST, pageName, type));
}
async function writeBuildManifest(distDir, buildManifests, currentEntrypoints, rewrites) {
    const buildManifest = mergeBuildManifests(buildManifests.values());
    const buildManifestPath = join(distDir, BUILD_MANIFEST);
    const middlewareBuildManifestPath = join(distDir, "server", `${MIDDLEWARE_BUILD_MANIFEST}.js`);
    const interceptionRewriteManifestPath = join(distDir, "server", `${INTERCEPTION_ROUTE_REWRITE_MANIFEST}.js`);
    deleteCache(buildManifestPath);
    deleteCache(middlewareBuildManifestPath);
    deleteCache(interceptionRewriteManifestPath);
    await writeFileAtomic(buildManifestPath, JSON.stringify(buildManifest, null, 2));
    await writeFileAtomic(middlewareBuildManifestPath, `self.__BUILD_MANIFEST=${JSON.stringify(buildManifest)};`);
    const interceptionRewrites = JSON.stringify(rewrites.beforeFiles.filter(isInterceptionRouteRewrite));
    await writeFileAtomic(interceptionRewriteManifestPath, `self.__INTERCEPTION_ROUTE_REWRITE_MANIFEST=${JSON.stringify(interceptionRewrites)};`);
    const content = {
        __rewrites: rewrites ? normalizeRewritesForBuildManifest(rewrites) : {
            afterFiles: [],
            beforeFiles: [],
            fallback: []
        },
        ...Object.fromEntries([
            ...currentEntrypoints.keys()
        ].map((pathname)=>[
                pathname,
                `static/chunks/pages${pathname === "/" ? "/index" : pathname}.js`
            ])),
        sortedPages: [
            ...currentEntrypoints.keys()
        ]
    };
    const buildManifestJs = `self.__BUILD_MANIFEST = ${JSON.stringify(content)};self.__BUILD_MANIFEST_CB && self.__BUILD_MANIFEST_CB()`;
    await writeFileAtomic(join(distDir, "static", "development", "_buildManifest.js"), buildManifestJs);
    await writeFileAtomic(join(distDir, "static", "development", "_ssgManifest.js"), srcEmptySsgManifest);
}
async function writeFallbackBuildManifest(distDir, buildManifests) {
    const fallbackBuildManifest = mergeBuildManifests([
        buildManifests.get("_app"),
        buildManifests.get("_error")
    ].filter(Boolean));
    const fallbackBuildManifestPath = join(distDir, `fallback-${BUILD_MANIFEST}`);
    deleteCache(fallbackBuildManifestPath);
    await writeFileAtomic(fallbackBuildManifestPath, JSON.stringify(fallbackBuildManifest, null, 2));
}
async function writeAppBuildManifest(distDir, appBuildManifests) {
    const appBuildManifest = mergeAppBuildManifests(appBuildManifests.values());
    const appBuildManifestPath = join(distDir, APP_BUILD_MANIFEST);
    deleteCache(appBuildManifestPath);
    await writeFileAtomic(appBuildManifestPath, JSON.stringify(appBuildManifest, null, 2));
}
async function writePagesManifest(distDir, pagesManifests) {
    const pagesManifest = mergePagesManifests(pagesManifests.values());
    const pagesManifestPath = join(distDir, "server", PAGES_MANIFEST);
    deleteCache(pagesManifestPath);
    await writeFileAtomic(pagesManifestPath, JSON.stringify(pagesManifest, null, 2));
}
async function writeAppPathsManifest(distDir, appPathsManifests) {
    const appPathsManifest = mergePagesManifests(appPathsManifests.values());
    const appPathsManifestPath = join(distDir, "server", APP_PATHS_MANIFEST);
    deleteCache(appPathsManifestPath);
    await writeFileAtomic(appPathsManifestPath, JSON.stringify(appPathsManifest, null, 2));
}
async function writeMiddlewareManifest(distDir, middlewareManifests) {
    const middlewareManifest = mergeMiddlewareManifests(middlewareManifests.values());
    const middlewareManifestPath = join(distDir, "server", MIDDLEWARE_MANIFEST);
    deleteCache(middlewareManifestPath);
    await writeFileAtomic(middlewareManifestPath, JSON.stringify(middlewareManifest, null, 2));
}
async function writeActionManifest(distDir, actionManifests) {
    const actionManifest = await mergeActionManifests(actionManifests.values());
    const actionManifestJsonPath = join(distDir, "server", `${SERVER_REFERENCE_MANIFEST}.json`);
    const actionManifestJsPath = join(distDir, "server", `${SERVER_REFERENCE_MANIFEST}.js`);
    const json = JSON.stringify(actionManifest, null, 2);
    deleteCache(actionManifestJsonPath);
    deleteCache(actionManifestJsPath);
    await writeFile(actionManifestJsonPath, json, "utf-8");
    await writeFile(actionManifestJsPath, `self.__RSC_SERVER_MANIFEST=${JSON.stringify(json)}`, "utf-8");
}
async function writeFontManifest(distDir, fontManifests) {
    const fontManifest = mergeFontManifests(fontManifests.values());
    const json = JSON.stringify(fontManifest, null, 2);
    const fontManifestJsonPath = join(distDir, "server", `${NEXT_FONT_MANIFEST}.json`);
    const fontManifestJsPath = join(distDir, "server", `${NEXT_FONT_MANIFEST}.js`);
    deleteCache(fontManifestJsonPath);
    deleteCache(fontManifestJsPath);
    await writeFileAtomic(fontManifestJsonPath, json);
    await writeFileAtomic(fontManifestJsPath, `self.__NEXT_FONT_MANIFEST=${JSON.stringify(json)}`);
}
async function writeLoadableManifest(distDir, loadableManifests) {
    const loadableManifest = mergeLoadableManifests(loadableManifests.values());
    const loadableManifestPath = join(distDir, REACT_LOADABLE_MANIFEST);
    const middlewareloadableManifestPath = join(distDir, "server", `${MIDDLEWARE_REACT_LOADABLE_MANIFEST}.js`);
    const json = JSON.stringify(loadableManifest, null, 2);
    deleteCache(loadableManifestPath);
    deleteCache(middlewareloadableManifestPath);
    await writeFileAtomic(loadableManifestPath, json);
    await writeFileAtomic(middlewareloadableManifestPath, `self.__REACT_LOADABLE_MANIFEST=${JSON.stringify(json)}`);
}
export async function writeManifests({ rewrites, distDir, buildManifests, appBuildManifests, pagesManifests, appPathsManifests, middlewareManifests, actionManifests, fontManifests, loadableManifests, currentEntrypoints }) {
    await writeBuildManifest(distDir, buildManifests, currentEntrypoints, rewrites);
    await writeAppBuildManifest(distDir, appBuildManifests);
    await writePagesManifest(distDir, pagesManifests);
    await writeAppPathsManifest(distDir, appPathsManifests);
    await writeMiddlewareManifest(distDir, middlewareManifests);
    await writeActionManifest(distDir, actionManifests);
    await writeFontManifest(distDir, fontManifests);
    await writeLoadableManifest(distDir, loadableManifests);
    await writeFallbackBuildManifest(distDir, buildManifests);
}
class ModuleBuildError extends Error {
}
function issueKey(issue) {
    return [
        issue.severity,
        issue.filePath,
        JSON.stringify(issue.title),
        JSON.stringify(issue.description)
    ].join("-");
}
export function formatIssue(issue) {
    const { filePath, title, description, source } = issue;
    let { documentationLink } = issue;
    let formattedTitle = renderStyledStringToErrorAnsi(title).replace(/\n/g, "\n    ");
    // TODO: Use error codes to identify these
    // TODO: Generalize adapting Turbopack errors to Next.js errors
    if (formattedTitle.includes("Module not found")) {
        // For compatiblity with webpack
        // TODO: include columns in webpack errors.
        documentationLink = "https://nextjs.org/docs/messages/module-not-found";
    }
    let formattedFilePath = filePath.replace("[project]/", "./").replaceAll("/./", "/").replace("\\\\?\\", "");
    let message;
    if (source && source.range) {
        const { start } = source.range;
        message = `${formattedFilePath}:${start.line + 1}:${start.column + 1}\n${formattedTitle}`;
    } else if (formattedFilePath) {
        message = `${formattedFilePath}\n${formattedTitle}`;
    } else {
        message = formattedTitle;
    }
    message += "\n";
    if ((source == null ? void 0 : source.range) && source.source.content) {
        const { start, end } = source.range;
        const { codeFrameColumns } = require("next/dist/compiled/babel/code-frame");
        message += codeFrameColumns(source.source.content, {
            start: {
                line: start.line + 1,
                column: start.column + 1
            },
            end: {
                line: end.line + 1,
                column: end.column + 1
            }
        }, {
            forceColor: true
        }).trim() + "\n\n";
    }
    if (description) {
        message += renderStyledStringToErrorAnsi(description) + "\n\n";
    }
    // TODO: make it possible to enable this for debugging, but not in tests.
    // if (detail) {
    //   message += renderStyledStringToErrorAnsi(detail) + '\n\n'
    // }
    // TODO: Include a trace from the issue.
    if (documentationLink) {
        message += documentationLink + "\n\n";
    }
    return message;
}
export function processIssues(currentIssues, name, result, throwIssue = false) {
    const newIssues = new Map();
    currentIssues.set(name, newIssues);
    const relevantIssues = new Set();
    for (const issue of result.issues){
        if (issue.severity !== "error" && issue.severity !== "fatal") continue;
        const key = issueKey(issue);
        const formatted = formatIssue(issue);
        newIssues.set(key, issue);
        // We show errors in node_modules to the console, but don't throw for them
        if (/(^|\/)node_modules(\/|$)/.test(issue.filePath)) continue;
        relevantIssues.add(formatted);
    }
    if (relevantIssues.size && throwIssue) {
        throw new ModuleBuildError([
            ...relevantIssues
        ].join("\n\n"));
    }
}
export function renderStyledStringToErrorAnsi(string) {
    function decodeMagicIdentifiers(str) {
        return str.replaceAll(MAGIC_IDENTIFIER_REGEX, (ident)=>{
            try {
                return magenta(`{${decodeMagicIdentifier(ident)}}`);
            } catch (e) {
                return magenta(`{${ident} (decoding failed: ${e})}`);
            }
        });
    }
    switch(string.type){
        case "text":
            return decodeMagicIdentifiers(string.value);
        case "strong":
            return bold(red(decodeMagicIdentifiers(string.value)));
        case "code":
            return green(decodeMagicIdentifiers(string.value));
        case "line":
            return string.value.map(renderStyledStringToErrorAnsi).join("");
        case "stack":
            return string.value.map(renderStyledStringToErrorAnsi).join("\n");
        default:
            throw new Error("Unknown StyledString type", string);
    }
}
const MILLISECONDS_IN_NANOSECOND = 1000000;
export function msToNs(ms) {
    return BigInt(Math.floor(ms)) * BigInt(MILLISECONDS_IN_NANOSECOND);
}
export async function handleRouteType({ rewrites, distDir, globalEntrypoints, currentIssues, buildManifests, appBuildManifests, pagesManifests, appPathsManifests, middlewareManifests, actionManifests, fontManifests, loadableManifests, currentEntrypoints, handleRequireCacheClearing, changeSubscription, readyIds, page, route }) {
    switch(route.type){
        case "page":
            {
                try {
                    if (globalEntrypoints.app) {
                        const writtenEndpoint = await globalEntrypoints.app.writeToDisk();
                        handleRequireCacheClearing == null ? void 0 : handleRequireCacheClearing("_app", writtenEndpoint);
                        processIssues(currentIssues, "_app", writtenEndpoint);
                    }
                    await loadBuildManifest(distDir, buildManifests, "_app");
                    await loadPagesManifest(distDir, pagesManifests, "_app");
                    if (globalEntrypoints.document) {
                        const writtenEndpoint = await globalEntrypoints.document.writeToDisk();
                        handleRequireCacheClearing == null ? void 0 : handleRequireCacheClearing("_document", writtenEndpoint);
                        processIssues(currentIssues, "_document", writtenEndpoint);
                    }
                    await loadPagesManifest(distDir, pagesManifests, "_document");
                    const writtenEndpoint = await route.htmlEndpoint.writeToDisk();
                    handleRequireCacheClearing == null ? void 0 : handleRequireCacheClearing(page, writtenEndpoint);
                    const type = writtenEndpoint == null ? void 0 : writtenEndpoint.type;
                    await loadBuildManifest(distDir, buildManifests, page);
                    await loadPagesManifest(distDir, pagesManifests, page);
                    if (type === "edge") {
                        await loadMiddlewareManifest(distDir, middlewareManifests, page, "pages");
                    } else {
                        middlewareManifests.delete(page);
                    }
                    await loadFontManifest(distDir, fontManifests, page, "pages");
                    await loadLoadableManifest(distDir, loadableManifests, page, "pages");
                    await writeManifests({
                        rewrites,
                        distDir,
                        buildManifests,
                        appBuildManifests,
                        pagesManifests,
                        appPathsManifests,
                        middlewareManifests,
                        actionManifests,
                        fontManifests,
                        loadableManifests,
                        currentEntrypoints
                    });
                    processIssues(currentIssues, page, writtenEndpoint);
                } finally{
                    changeSubscription == null ? void 0 : changeSubscription(page, "server", false, route.dataEndpoint, (pageName)=>{
                        // Report the next compilation again
                        readyIds.delete(page);
                        return {
                            event: HMR_ACTIONS_SENT_TO_BROWSER.SERVER_ONLY_CHANGES,
                            pages: [
                                pageName
                            ]
                        };
                    });
                    changeSubscription == null ? void 0 : changeSubscription(page, "client", false, route.htmlEndpoint, ()=>{
                        return {
                            event: HMR_ACTIONS_SENT_TO_BROWSER.CLIENT_CHANGES
                        };
                    });
                    if (globalEntrypoints.document) {
                        changeSubscription == null ? void 0 : changeSubscription("_document", "server", false, globalEntrypoints.document, ()=>{
                            return {
                                action: HMR_ACTIONS_SENT_TO_BROWSER.RELOAD_PAGE
                            };
                        });
                    }
                }
                break;
            }
        case "page-api":
            {
                const writtenEndpoint = await route.endpoint.writeToDisk();
                handleRequireCacheClearing == null ? void 0 : handleRequireCacheClearing(page, writtenEndpoint);
                const type = writtenEndpoint == null ? void 0 : writtenEndpoint.type;
                await loadPagesManifest(distDir, pagesManifests, page);
                if (type === "edge") {
                    await loadMiddlewareManifest(distDir, middlewareManifests, page, "pages");
                } else {
                    middlewareManifests.delete(page);
                }
                await loadLoadableManifest(distDir, loadableManifests, page, "pages");
                await writeManifests({
                    rewrites,
                    distDir,
                    buildManifests,
                    appBuildManifests,
                    pagesManifests,
                    appPathsManifests,
                    middlewareManifests,
                    actionManifests,
                    fontManifests,
                    loadableManifests,
                    currentEntrypoints
                });
                processIssues(currentIssues, page, writtenEndpoint);
                break;
            }
        case "app-page":
            {
                const writtenEndpoint = await route.htmlEndpoint.writeToDisk();
                handleRequireCacheClearing == null ? void 0 : handleRequireCacheClearing(page, writtenEndpoint);
                changeSubscription == null ? void 0 : changeSubscription(page, "server", true, route.rscEndpoint, (_page, change)=>{
                    if (change.issues.some((issue)=>issue.severity === "error")) {
                        // Ignore any updates that has errors
                        // There will be another update without errors eventually
                        return;
                    }
                    // Report the next compilation again
                    readyIds.delete(page);
                    return {
                        action: HMR_ACTIONS_SENT_TO_BROWSER.SERVER_COMPONENT_CHANGES
                    };
                });
                const type = writtenEndpoint == null ? void 0 : writtenEndpoint.type;
                if (type === "edge") {
                    await loadMiddlewareManifest(distDir, middlewareManifests, page, "app");
                } else {
                    middlewareManifests.delete(page);
                }
                await loadAppBuildManifest(distDir, appBuildManifests, page);
                await loadBuildManifest(distDir, buildManifests, page, "app");
                await loadAppPathManifest(distDir, appPathsManifests, page, "app");
                await loadActionManifest(distDir, actionManifests, page);
                await loadFontManifest(distDir, fontManifests, page, "app");
                await writeManifests({
                    rewrites,
                    distDir,
                    buildManifests,
                    appBuildManifests,
                    pagesManifests,
                    appPathsManifests,
                    middlewareManifests,
                    actionManifests,
                    fontManifests,
                    loadableManifests,
                    currentEntrypoints
                });
                processIssues(currentIssues, page, writtenEndpoint, true);
                break;
            }
        case "app-route":
            {
                const writtenEndpoint = await route.endpoint.writeToDisk();
                handleRequireCacheClearing == null ? void 0 : handleRequireCacheClearing(page, writtenEndpoint);
                const type = writtenEndpoint == null ? void 0 : writtenEndpoint.type;
                await loadAppPathManifest(distDir, appPathsManifests, page, "app-route");
                if (type === "edge") {
                    await loadMiddlewareManifest(distDir, middlewareManifests, page, "app-route");
                } else {
                    middlewareManifests.delete(page);
                }
                await writeManifests({
                    rewrites,
                    distDir,
                    buildManifests,
                    appBuildManifests,
                    pagesManifests,
                    appPathsManifests,
                    middlewareManifests,
                    actionManifests,
                    fontManifests,
                    loadableManifests,
                    currentEntrypoints
                });
                processIssues(currentIssues, page, writtenEndpoint, true);
                break;
            }
        default:
            {
                throw new Error(`unknown route type ${route.type} for ${page}`);
            }
    }
}

//# sourceMappingURL=turbopack-utils.js.map